@@ -227,7 +227,6 @@ VRY_Prep(struct req *req)
 	if (req->hash_objhead == NULL) {
 		/* Not a waiting list return */
 		AZ(req->vary_b);
-		AZ(req->vary_l);
 		AZ(req->vary_e);
 		(void)WS_ReserveAll(req->ws);
 	} else {
@@ -248,7 +247,6 @@ VRY_Clear(struct req *req)
 		free(req->vary_b);
 	req->vary_b = NULL;
 	AZ(req->vary_e);
-	AZ(req->vary_l);
 }
 
 /**********************************************************************
@@ -259,9 +257,9 @@ void
 VRY_Finish(struct req *req, enum vry_finish_flag flg)
 {
 	uint8_t *p = NULL;
+	size_t l;
 
 	if (req->vary_b + 2 >= req->vary_e) {
-		AZ(req->vary_l);
 		req->vary_b = NULL;
 		req->vary_e = NULL;
 		WS_Release(req->ws, 0);
@@ -269,16 +267,13 @@ VRY_Finish(struct req *req, enum vry_finish_flag flg)
 		return;
 	}
 
-	(void)VRY_Validate(req->vary_b);
-	if (flg == KEEP && req->vary_l != NULL) {
-		p = malloc(req->vary_l - req->vary_b);
-		if (p != NULL) {
-			memcpy(p, req->vary_b, req->vary_l - req->vary_b);
-			(void)VRY_Validate(p);
-		}
+	l = VRY_Validate(req->vary_b);
+	if (flg == KEEP && l > 3) {
+		p = malloc(l);
+		if (p != NULL)
+			memcpy(p, req->vary_b, l);
 	}
 	WS_Release(req->ws, 0);
-	req->vary_l = NULL;
 	req->vary_e = NULL;
 	req->vary_b = p;
 }
@@ -291,7 +286,7 @@ VRY_Finish(struct req *req, enum vry_finish_flag flg)
  */
 
 int
-VRY_Match(struct req *req, const uint8_t *vary)
+VRY_Match(const struct req *req, const uint8_t *vary)
 {
 	uint8_t *vsp = req->vary_b;
 	const char *h, *e;
@@ -343,11 +338,10 @@ VRY_Match(struct req *req, const uint8_t *vary)
 			memcpy(vsp + 2, vary + 2, vary[2] + 2);
 			if (h != NULL)
 				memcpy(vsp + 2 + vsp[2] + 2, h, lh);
-			vsp[ln] = 0xff;
-			vsp[ln + 1] = 0xff;
-			vsp[ln + 2] = 0;
-			(void)VRY_Validate(vsp);
-			req->vary_l = vsp + ln + 3;
+			vsp[ln++] = 0xff;
+			vsp[ln++] = 0xff;
+			vsp[ln++] = 0;
+			assert(VRY_Validate(vsp) == ln);
 
 			i = vry_cmp(vary, vsp);
 			assert(i == 0 || i == 2);
@@ -363,7 +357,6 @@ VRY_Match(struct req *req, const uint8_t *vary)
 	}
 	if (oflo) {
 		vsp = req->vary_b;
-		req->vary_l = NULL;
 		if (vsp + 2 < req->vary_e) {
 			vsp[0] = 0xff;
 			vsp[1] = 0xff;
@@ -382,12 +375,13 @@ VRY_Match(struct req *req, const uint8_t *vary)
 static unsigned
 VRY_Validate(const uint8_t *vary)
 {
-	unsigned retval = 0;
+	unsigned l, retval = 0;
 
 	while (vary[2] != 0) {
-		assert(strlen((const char*)vary+3) == vary[2]);
-		retval += VRY_Len(vary);
-		vary += VRY_Len(vary);
+		assert(strlen((const char*)vary + 3) == vary[2]);
+		l = VRY_Len(vary);
+		retval += l;
+		vary += l;
 	}
 	return (retval + 3);
 }
