@@ -63,10 +63,6 @@ static const struct cli_cmd_desc *cmds[] = {
 #include "tbl/cli_cmds.h"
 };
 
-#ifdef GCOVING
-    int __gcov_flush(void);
-#endif
-
 static const int ncmds = sizeof cmds / sizeof cmds[0];
 
 static int		cli_i = -1, cli_o = -1;
@@ -112,9 +108,7 @@ mcf_panic(struct cli *cli, const char * const *av, void *priv)
 	(void)cli;
 	(void)av;
 	(void)priv;
-#ifdef GCOVING
-	__gcov_flush();
-#endif
+	v_gcov_flush();
 	AZ(strcmp("", "You asked for it"));
 	/* NOTREACHED */
 	abort();
@@ -219,12 +213,17 @@ mgt_cli_askchild(unsigned *status, char **resp, const char *fmt, ...)
 
 /*--------------------------------------------------------------------*/
 
-void
-mgt_cli_start_child(int fdi, int fdo)
+unsigned
+mgt_cli_start_child(int fd, double tmo)
 {
+	unsigned u;
 
-	cli_i = fdi;
-	cli_o = fdo;
+	cli_i = fd;
+	cli_o = fd;
+	if (VCLI_ReadResult(cli_i, &u, NULL, tmo)) {
+		return (CLIS_COMMS);
+	}
+	return (u);
 }
 
 /*--------------------------------------------------------------------*/
@@ -281,7 +280,7 @@ mcf_auth(struct cli *cli, const char *const *av, void *priv)
 	fd = open(secret_file, O_RDONLY);
 	if (fd < 0) {
 		VCLI_Out(cli, "Cannot open secret file (%s)\n",
-		    vstrerror(errno));
+		    VAS_errtxt(errno));
 		VCLI_SetResult(cli, CLIS_CANT);
 		VJ_master(JAIL_MASTER_LOW);
 		return;
@@ -347,11 +346,13 @@ mgt_cli_cb_after(const struct cli *cli)
 
 	MGT_Complain(C_CLI, "CLI %s Wr %03u %s",
 	    cli->ident, cli->result, VSB_data(cli->sb));
-	if (cli->priv == stderr &&
-	    cli->result != CLIS_OK && *VSB_data(cli->cmd) != '-') {
-		MGT_Complain(C_ERR, "-I file CLI command failed (%d)\n%s\n",
+	if (cli->priv != stderr)
+		return;
+	if (cli->result == CLIS_TRUNCATED)
+		ARGV_ERR("-I file had incomplete CLI command at the end\n");
+	if (cli->result != CLIS_OK && *VSB_data(cli->cmd) != '-') {
+		ARGV_ERR("-I file CLI command failed (%d)\n%s\n",
 		    cli->result, VSB_data(cli->sb));
-		exit(2);
 	}
 }
 
@@ -455,13 +456,10 @@ static int
 telnet_accept(const struct vev *ev, int what)
 {
 	struct vsb *vsb;
-	struct sockaddr_storage addr;
-	socklen_t addrlen;
 	int i;
 
 	(void)what;
-	addrlen = sizeof addr;
-	i = accept(ev->fd, (void *)&addr, &addrlen);
+	i = accept(ev->fd, NULL, NULL);
 	if (i < 0 && errno == EBADF)
 		return (1);
 	if (i < 0)
@@ -559,7 +557,7 @@ mgt_cli_telnet(const char *T_arg)
 struct m_addr {
 	unsigned		magic;
 #define M_ADDR_MAGIC		0xbc6217ed
-	struct suckaddr		*sa;
+	const struct suckaddr	*sa;
 	VTAILQ_ENTRY(m_addr)	list;
 };
 
@@ -591,7 +589,7 @@ Marg_connect(const struct vev *e, int what)
 	M_fd = VTCP_connected(M_fd);
 	if (M_fd < 0) {
 		MGT_Complain(C_INFO, "Could not connect to CLI-master: %s",
-			vstrerror(errno));
+			VAS_errtxt(errno));
 		ma = VTAILQ_FIRST(&m_addr_list);
 		AN(ma);
 		VTAILQ_REMOVE(&m_addr_list, ma, list);
