@@ -51,7 +51,7 @@
 #include "hash/hash_slinger.h"
 #include "libvcc.h"
 #include "vcli_serve.h"
-#include "vend.h"
+#include "vct.h"
 #include "vev.h"
 #include "vfil.h"
 #include "vin.h"
@@ -62,6 +62,7 @@
 #include "vtim.h"
 #include "waiter/mgt_waiter.h"
 #include "vsa.h"
+#include "vus.h"
 
 struct heritage		heritage;
 unsigned		d_flag = 0;
@@ -74,14 +75,17 @@ static char		*workdir;
 
 static struct vfil_path *vcl_path = NULL;
 
-static const char opt_spec[] = "?a:b:Cdf:Fh:i:I:j:l:M:n:P:p:r:S:s:T:t:VW:x:";
+static const char opt_spec[] = "?a:b:CdE:f:Fh:i:I:j:l:M:n:P:p:r:S:s:T:t:VW:x:";
 
 /*--------------------------------------------------------------------*/
 
 static void
 usage(void)
 {
-#define FMT "  %-28s # %s\n"
+	char *p;
+
+#define FMT_NONL "  %-28s # %s"
+#define FMT FMT_NONL "\n"
 
 	printf( "Usage: varnishd [options]\n");
 
@@ -103,6 +107,11 @@ usage(void)
 	printf(FMT, "", "Can be specified multiple times.");
 	printf(FMT, "-n dir", "Working directory");
 
+	p = VIN_n_Arg(NULL);
+	AN(p);
+	printf(FMT_NONL "  default: %s\n", "", "", p);
+	free(p);
+
 	printf("\n-b can be used only once, and not together with -f\n");
 
 	printf("\nDocumentation options:\n");
@@ -123,6 +132,7 @@ usage(void)
 	printf(FMT, "-P file", "PID file");
 	printf(FMT, "-i identity", "Identity of varnish instance");
 	printf(FMT, "-I clifile", "Initialization CLI commands");
+	printf(FMT, "-E extension", "Load extension");
 
 	printf("\nTuning options:\n");
 
@@ -284,6 +294,10 @@ mgt_Cflag_atexit(void)
 	/* Only master process */
 	if (getpid() != heritage.mgt_pid)
 		return;
+	if (arg_list_count("E")) {
+		vext_cleanup(1);
+		VJ_rmdir("vext_cache");
+	}
 	VJ_rmdir("vmod_cache");
 	(void)chdir("/");
 	VJ_rmdir(workdir);
@@ -480,10 +494,10 @@ mgt_b_conv(const char *b_arg)
 	VSB_cat(vsb, "backend default ");
 	if (!strcasecmp(b_arg, "none"))
 		VSB_cat(vsb, "none;\n");
-	else if (*b_arg != '/')
-		VSB_printf(vsb, "{\n    .host = \"%s\";\n}\n", b_arg);
-	else
+	else if (VUS_is(b_arg))
 		VSB_printf(vsb, "{\n    .path = \"%s\";\n}\n", b_arg);
+	else
+		VSB_printf(vsb, "{\n    .host = \"%s\";\n}\n", b_arg);
 	AZ(VSB_finish(vsb));
 	fa->src = strdup(VSB_data(vsb));
 	AN(fa->src);
@@ -582,6 +596,7 @@ main(int argc, char * const *argv)
 	const char *S_arg = NULL;
 	const char *s_arg = "default,100m";
 	const char *W_arg = NULL;
+	const char *c;
 	char *p;
 	struct cli cli[1];
 	const char *err;
@@ -660,6 +675,7 @@ main(int argc, char * const *argv)
 			W_arg = optarg;
 			break;
 		case 'a':
+		case 'E':
 		case 'f':
 		case 'I':
 		case 'p':
@@ -752,6 +768,9 @@ main(int argc, char * const *argv)
 			if (*alp->val != '\0')
 				alp->priv = mgt_f_read(alp->val);
 			break;
+		case 'E':
+			vext_argument(alp->val);
+			break;
 		case 'I':
 			VJ_master(JAIL_MASTER_FILE);
 			I_fd = open(alp->val, O_RDONLY);
@@ -809,6 +828,10 @@ main(int argc, char * const *argv)
 
 	if (i_arg == NULL || *i_arg == '\0')
 		i_arg = mgt_HostName();
+	else for (c = i_arg; *c != '\0'; c++) {
+		if (!vct_istchar(*c))
+			ARGV_ERR("Invalid character '%c' for -i\n", *c);
+	}
 	heritage.identity = i_arg;
 
 	mgt_ProcTitle("Mgt");
@@ -821,6 +844,7 @@ main(int argc, char * const *argv)
 
 	VJ_master(JAIL_MASTER_SYSTEM);
 	AZ(system("rm -rf vmod_cache"));
+	AZ(system("rm -rf vext_cache"));
 	VJ_master(JAIL_MASTER_LOW);
 
 	if (VJ_make_subdir("vmod_cache", "VMOD cache", NULL)) {
@@ -829,6 +853,13 @@ main(int argc, char * const *argv)
 		    workdir, VAS_errtxt(errno));
 	}
 
+	if (arg_list_count("E") &&
+	    VJ_make_subdir("vext_cache", "VMOD cache", NULL)) {
+		ARGV_ERR(
+		    "Cannot create vmod directory (%s/vext_cache): %s\n",
+		    workdir, VAS_errtxt(errno));
+	}
+
 	if (C_flag)
 		AZ(atexit(mgt_Cflag_atexit));
 
@@ -836,11 +867,13 @@ main(int argc, char * const *argv)
 	if (!arg_list_count("s"))
 		STV_Config(s_arg);
 
-	/* Configure Transient storage, if user did not */
-	STV_Config_Transient();
+	/* Configure CLI and Transient storage, if user did not */
+	STV_Config_Final();
 
 	mgt_vcl_init();
 
+	vext_copyin(vident);
+
 	u = 0;
 	VTAILQ_FOREACH(alp, &arglist, list) {
 		if (!strcmp(alp->arg, "f") && alp->priv != NULL)
@@ -968,6 +1001,11 @@ main(int argc, char * const *argv)
 	MGT_Complain(C_INFO, "manager dies");
 	mgt_cli_close_all();
 	VEV_Destroy(&mgt_evb);
+	VJ_master(JAIL_MASTER_SYSTEM);
+	/*lint -e(730)*/
+	vext_cleanup(! MGT_DO_DEBUG(DBG_VMOD_SO_KEEP));
+	(void)rmdir("vext_cache");
+	VJ_master(JAIL_MASTER_LOW);
 	VTAILQ_FOREACH(alp, &arglist, list) {
 		if (!strcmp(alp->arg, "P"))
 			VPF_Remove(alp->priv);
