@@ -62,6 +62,10 @@ static const int tab0 = 3;
 
 /*--------------------------------------------------------------------*/
 
+static const char TYPE_TIMEOUT_TEXT[] =
+	"\n\n"
+	"NB: This parameter can be disabled with the value \"never\".";
+
 static const char OBJ_STICKY_TEXT[] =
 	"\n\n"
 	"NB: This parameter is evaluated only when objects are created. "
@@ -337,6 +341,8 @@ mcf_param_show(struct cli *cli, const char * const *av, void *priv)
 				    margin1, "", pp->max);
 			VCLI_Out(cli, "\n");
 			mcf_wrap(cli, pp->descr);
+			if (pp->func == tweak_timeout)
+				mcf_wrap(cli, TYPE_TIMEOUT_TEXT);
 			if (pp->flags & OBJ_STICKY)
 				mcf_wrap(cli, OBJ_STICKY_TEXT);
 			if (pp->flags & DELAYED_EFFECT)
@@ -610,16 +616,25 @@ mcf_param_reset(struct cli *cli, const char * const *av, void *priv)
 }
 
 /*--------------------------------------------------------------------
- * Add a group of parameters to the global set and sort by name.
+ * Initialize parameters and sort by name.
  */
 
-void
-MCF_AddParams(struct parspec *ps)
+static struct parspec mgt_parspec[] = {
+#define PARAM_ALL
+#define PARAM_PRE {
+#define PARAM(typ, fld, nm, ...) #nm, __VA_ARGS__
+#define PARAM_POST },
+#include "tbl/params.h"
+	{ NULL, NULL, NULL }
+};
+
+static void
+mcf_init_params(void)
 {
 	struct parspec *pp;
 	const char *s;
 
-	for (pp = ps; pp->name != NULL; pp++) {
+	for (pp = mgt_parspec; pp->name != NULL; pp++) {
 		AN(pp->func);
 		s = strchr(pp->descr, '\0');
 		if (isspace(s[-1])) {
@@ -704,7 +719,7 @@ mcf_wash_param(struct cli *cli, struct parspec *pp, enum mcf_which_e which,
 static struct cli_proto cli_params[] = {
 	{ CLICMD_PARAM_SHOW,	"", mcf_param_show, mcf_param_show_json },
 	{ CLICMD_PARAM_SET,	"", mcf_param_set, mcf_param_set_json },
-	{ CLICMD_PARAM_RESET,	"", mcf_param_reset },
+	{ CLICMD_PARAM_RESET,	"", mcf_param_reset, mcf_param_set_json },
 	{ NULL }
 };
 
@@ -720,9 +735,7 @@ MCF_InitParams(struct cli *cli)
 	struct vsb *vsb;
 	ssize_t def, low;
 
-	MCF_AddParams(mgt_parspec);
-	MCF_AddParams(VSL_parspec);
-
+	mcf_init_params();
 	MCF_TcpParams();
 
 	def = 80 * 1024;
@@ -818,6 +831,7 @@ MCF_DumpRstParam(void)
 	struct plist *pl;
 	const struct parspec *pp;
 	const char *p, *q, *t1, *t2;
+	unsigned flags;
 	size_t z;
 
 	printf("\n.. The following is the autogenerated "
@@ -850,40 +864,42 @@ MCF_DumpRstParam(void)
 		MCF_DYN_REASON(Minimum, min);
 		MCF_DYN_REASON(Maximum, max);
 #undef MCF_DYN_REASON
-		/*
-		 * XXX: we should mark the params with one/two flags
-		 * XXX: that say if ->min/->max are valid, so we
-		 * XXX: can emit those also in help texts.
-		 */
-		if (pp->flags & ~DOCS_FLAGS) {
+		flags = pp->flags & ~DOCS_FLAGS;
+		if (pp->func == tweak_timeout)
+			flags |= TYPE_TIMEOUT;
+		if (flags) {
 			printf("\t* Flags: ");
 			q = "";
 
-			if (pp->flags & DELAYED_EFFECT) {
+			if (flags & TYPE_TIMEOUT) {
+				printf("%stimeout", q);
+				q = ", ";
+			}
+			if (flags & DELAYED_EFFECT) {
 				printf("%sdelayed", q);
 				q = ", ";
 			}
-			if (pp->flags & MUST_RESTART) {
+			if (flags & MUST_RESTART) {
 				printf("%smust_restart", q);
 				q = ", ";
 			}
-			if (pp->flags & MUST_RELOAD) {
+			if (flags & MUST_RELOAD) {
 				printf("%smust_reload", q);
 				q = ", ";
 			}
-			if (pp->flags & EXPERIMENTAL) {
+			if (flags & EXPERIMENTAL) {
 				printf("%sexperimental", q);
 				q = ", ";
 			}
-			if (pp->flags & WIZARD) {
+			if (flags & WIZARD) {
 				printf("%swizard", q);
 				q = ", ";
 			}
-			if (pp->flags & ONLY_ROOT) {
+			if (flags & ONLY_ROOT) {
 				printf("%sonly_root", q);
 				q = ", ";
 			}
-			if (pp->flags & OBJ_STICKY) {
+			if (flags & OBJ_STICKY) {
 				printf("%sobj_sticky", q);
 				q = ", ";
 			}
