@@ -58,7 +58,7 @@ STV_next(void)
 	static struct stevedore *stv;
 	struct stevedore *r;
 
-	AZ(pthread_mutex_lock(&stv_mtx));
+	PTOK(pthread_mutex_lock(&stv_mtx));
 	if (!STV__iter(&stv))
 		AN(STV__iter(&stv));
 	if (stv == stv_transient) {
@@ -66,7 +66,7 @@ STV_next(void)
 		AN(STV__iter(&stv));
 	}
 	r = stv;
-	AZ(pthread_mutex_unlock(&stv_mtx));
+	PTOK(pthread_mutex_unlock(&stv_mtx));
 	AN(r);
 	return (r);
 }
@@ -84,12 +84,15 @@ STV_NewObject(struct worker *wrk, struct objcore *oc,
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
-	assert(wsl > 0);
 
 	wrk->strangelove = cache_param->nuke_limit;
 	AN(stv->allocobj);
-	if (stv->allocobj(wrk, stv, oc, wsl) == 0)
+	if (stv->allocobj(wrk, stv, oc, wsl) == 0) {
+		VSLb(wrk->vsl, SLT_Error,
+		    "Failed to create object object from %s %s",
+		    stv->name, stv->ident);
 		return (0);
+	}
 	oc->oa_present = 0;
 	wrk->stats->n_object++;
 	VSLb(wrk->vsl, SLT_Storage, "%s %s",
@@ -175,7 +178,7 @@ STV_open(void)
 	char buf[1024];
 
 	ASSERT_CLI();
-	AZ(pthread_mutex_init(&stv_mtx, &mtxattr_errorcheck));
+	PTOK(pthread_mutex_init(&stv_mtx, &mtxattr_errorcheck));
 
 	/* This string was prepared for us before the fork, and should
 	 * point to a configured stevedore. */
@@ -303,6 +306,8 @@ VRT_stevedore_##nm(VCL_STEVEDORE stv)			\
 {							\
 	if (stv == NULL)				\
 		return (0);				\
+	if (stv->var_##nm == NULL)			\
+		return (dval);				\
 	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);	\
 	return (stv->var_##nm(stv));			\
 }
