@@ -44,6 +44,9 @@
 /* Flags for allocating memory in sml_stv_alloc */
 #define LESS_MEM_ALLOCED_IS_OK	1
 
+// marker pointer for sml_trimstore
+static void *trim_once = &trim_once;
+
 /*-------------------------------------------------------------------*/
 
 static struct storage *
@@ -145,7 +148,7 @@ SML_allocobj(struct worker *wrk, const struct stevedore *stv,
 
 	AN(stv->sml_alloc);
 
-	ltot = sizeof(struct object) + PRNDUP(wsl);
+	ltot = sizeof(*o) + PRNDUP(wsl);
 
 	do {
 		st = stv->sml_alloc(stv, ltot);
@@ -249,24 +252,52 @@ sml_slim(struct worker *wrk, struct objcore *oc)
 	}
 }
 
+static void
+sml_bocfini(const struct stevedore *stv, struct boc *boc)
+{
+	struct storage *st;
+
+	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
+	CHECK_OBJ_NOTNULL(boc, BOC_MAGIC);
+
+	if (boc->stevedore_priv == NULL ||
+	    boc->stevedore_priv == trim_once)
+		return;
+
+	/* Free any leftovers from Trim */
+	TAKE_OBJ_NOTNULL(st, &boc->stevedore_priv, STORAGE_MAGIC);
+	sml_stv_free(stv, st);
+}
+
+/*
+ * called in two cases:
+ * - oc->boc == NULL: cache object on LRU freed
+ * - oc->boc != NULL: cache object replaced for backend error
+ */
 static void v_matchproto_(objfree_f)
 sml_objfree(struct worker *wrk, struct objcore *oc)
 {
+	const struct stevedore *stv;
 	struct storage *st;
 	struct object *o;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
-	sml_slim(wrk, oc);
+	stv = oc->stobj->stevedore;
+	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
 	CAST_OBJ_NOTNULL(o, oc->stobj->priv, OBJECT_MAGIC);
+
+	sml_slim(wrk, oc);
 	st = o->objstore;
 	CHECK_OBJ_NOTNULL(st, STORAGE_MAGIC);
 	FINI_OBJ(o);
 
-	if (oc->boc == NULL && oc->stobj->stevedore->lru != NULL)
+	if (oc->boc != NULL)
+		sml_bocfini(stv, oc->boc);
+	else if (stv->lru != NULL)
 		LRU_Remove(oc);
 
-	sml_stv_free(oc->stobj->stevedore, st);
+	sml_stv_free(stv, st);
 
 	memset(oc->stobj, 0, sizeof oc->stobj);
 
@@ -324,7 +355,7 @@ sml_iterator(struct worker *wrk, struct objcore *oc,
 	l = 0;
 
 	u = 0;
-	if (boc->len_so_far == 0) {
+	if (boc->fetched_so_far == 0) {
 		ret = func(priv, OBJ_ITER_FLUSH, NULL, 0);
 		if (ret)
 			return (ret);
@@ -445,6 +476,8 @@ sml_getspace(struct worker *wrk, struct objcore *oc, ssize_t *sz,
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	AN(sz);
 	AN(ptr);
+	if (*sz == 0)
+		*sz = cache_param->fetch_chunksize;
 	assert(*sz > 0);
 
 	o = sml_getobj(wrk, oc);
@@ -506,6 +539,10 @@ sml_trimstore(struct worker *wrk, struct objcore *oc)
 	stv = oc->stobj->stevedore;
 	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
 
+	if (oc->boc->stevedore_priv != NULL)
+		WRONG("sml_trimstore already called");
+	oc->boc->stevedore_priv = trim_once;
+
 	if (stv->sml_free == NULL)
 		return;
 
@@ -539,7 +576,6 @@ sml_trimstore(struct worker *wrk, struct objcore *oc)
 	VTAILQ_INSERT_HEAD(&o->list, st1, list);
 	Lck_Unlock(&oc->boc->mtx);
 	/* sml_bocdone frees this */
-	AZ(oc->boc->stevedore_priv);
 	oc->boc->stevedore_priv = st;
 }
 
@@ -547,7 +583,6 @@ static void v_matchproto_(objbocdone_f)
 sml_bocdone(struct worker *wrk, struct objcore *oc, struct boc *boc)
 {
 	const struct stevedore *stv;
-	struct storage *st;
 
 	CHECK_OBJ_NOTNULL(wrk, WORKER_MAGIC);
 	CHECK_OBJ_NOTNULL(oc, OBJCORE_MAGIC);
@@ -555,12 +590,7 @@ sml_bocdone(struct worker *wrk, struct objcore *oc, struct boc *boc)
 	stv = oc->stobj->stevedore;
 	CHECK_OBJ_NOTNULL(stv, STEVEDORE_MAGIC);
 
-	if (boc->stevedore_priv != NULL) {
-		/* Free any leftovers from Trim */
-		CAST_OBJ_NOTNULL(st, boc->stevedore_priv, STORAGE_MAGIC);
-		boc->stevedore_priv = NULL;
-		sml_stv_free(stv, st);
-	}
+	sml_bocfini(stv, boc);
 
 	if (stv->lru != NULL) {
 		if (isnan(wrk->lastused))
