@@ -32,7 +32,6 @@
  * - thank you, Marc Alexander Lehmann
  */
 
-//lint -e{766}
 #include "config.h"
 
 #if defined(HAVE_EPOLL_CTL)
@@ -134,7 +133,8 @@ vwe_thread(void *priv)
 			active = Wait_HeapDelete(w, wp);
 			Lck_Unlock(&vwe->mtx);
 			if (!active) {
-				VSL(SLT_Debug, wp->fd, "epoll: spurious event");
+				VSL(SLT_Debug, NO_VXID,
+				    "epoll: spurious event (%d)", wp->fd);
 				continue;
 			}
 			AZ(epoll_ctl(vwe->epfd, EPOLL_CTL_DEL, wp->fd, NULL));
@@ -201,7 +201,7 @@ vwe_init(struct waiter *w)
 	ee.data.ptr = vwe;
 	AZ(epoll_ctl(vwe->epfd, EPOLL_CTL_ADD, vwe->pipe[0], &ee));
 
-	AZ(pthread_create(&vwe->thread, NULL, vwe_thread, vwe));
+	PTOK(pthread_create(&vwe->thread, NULL, vwe_thread, vwe));
 }
 
 /*--------------------------------------------------------------------
@@ -221,7 +221,7 @@ vwe_fini(struct waiter *w)
 	vwe->die = 1;
 	assert(write(vwe->pipe[1], "Y", 1) == 1);
 	Lck_Unlock(&vwe->mtx);
-	AZ(pthread_join(vwe->thread, &vp));
+	PTOK(pthread_join(vwe->thread, &vp));
 	Lck_Delete(&vwe->mtx);
 }
 
