@@ -37,6 +37,7 @@
 CMD_GLOBAL(barrier)
 CMD_GLOBAL(delay)
 CMD_GLOBAL(shell)
+CMD_GLOBAL(include)
 #undef CMD_GLOBAL
 
 #ifndef CMD_TOP
@@ -44,6 +45,7 @@ CMD_GLOBAL(shell)
 #endif
 CMD_TOP(client)
 CMD_TOP(feature)
+CMD_TOP(filewrite)
 CMD_TOP(haproxy)
 #ifdef VTEST_WITH_VTC_LOGEXPECT
 CMD_TOP(logexpect)
