@@ -83,75 +83,40 @@ tokens = {
 #######################################################################
 # Our methods and actions
 
-returns = (
-    ###############################################################
-    # Client side
+returns = []
 
-    ('recv',
-     "C",
-     ('fail', 'synth', 'restart', 'pass', 'pipe', 'hash', 'purge', 'vcl')
-    ),
-    ('pipe',
-     "C",
-     ('fail', 'synth', 'pipe',)
-    ),
-    ('pass',
-     "C",
-     ('fail', 'synth', 'restart', 'fetch',)
-    ),
-    ('hash',
-     "C",
-     ('fail', 'lookup',)
-    ),
-    ('purge',
-     "C",
-     ('fail', 'synth', 'restart',)
-    ),
-    ('miss',
-     "C",
-     ('fail', 'synth', 'restart', 'pass', 'fetch',)
-    ),
-    ('hit',
-     "C",
-     ('fail', 'synth', 'restart', 'pass', 'deliver',)
-    ),
-    ('deliver',
-     "C",
-     ('fail', 'synth', 'restart', 'deliver',)
-    ),
-    ('synth',
-     "C",
-     ('fail', 'restart', 'deliver',)
-    ),
+def parse_subs(ln, c):
+    ctx = ['C', 'B', 'H']
+    rets = []
+    for i in ln[2:]:
+        if (len(i) > 4 and i.startswith('  | ``') and i.endswith('``')):
+            rets.append(i[6:-2].split('(')[0])
+    returns.append((ln[0][4:], ctx[c], rets))
 
-    ###############################################################
-    # Backend-fetch
+def parse_subs_doc(fn):
+    l = []
+    j = -1
+    start = False
+    for i in open(fn):
+        if i.startswith('##########'):
+            j += 1
+            l.append([])
+            start = True
+            continue
+        if start:
+            l[j].append(i.rstrip())
+    for i in range(0, len(l)):
+        for n in range(0, len(l[i])):
+            if len(l[i][n]) < 5 or not l[i][n].startswith("~~~~~"):
+                continue
+            m = n + 1
+            while m < len(l[i]) and (len(l[i][m]) < 5 or not l[i][m].startswith("~~~~~")):
+                m += 1
+            if m != len(l[i]):
+                m -= 2
+            parse_subs(l[i][n-1:m], i)
 
-    ('backend_fetch',
-     "B",
-     ('fail', 'fetch', 'abandon', 'error')
-    ),
-    ('backend_response',
-     "B",
-     ('fail', 'deliver', 'retry', 'abandon', 'pass', 'error')
-    ),
-    ('backend_error',
-     "B",
-     ('fail', 'deliver', 'retry', 'abandon')
-    ),
-
-    ###############################################################
-    # Housekeeping
-
-    ('init',
-     "H",
-     ('ok', 'fail')
-    ),
-    ('fini',
-     "H",
-     ('ok',)
-    ),
-)
+parse_subs_doc(join(srcroot, "doc/sphinx/reference/vcl_step.rst"))
 
 #######################################################################
 # Variables available in sessions
@@ -168,9 +133,18 @@ def varproto(s):
         fh.write(s + ";\n")
         varprotos[s] = True
 
+def var_is_wildcard(sym):
+    return sym[-2:] == '.*'
+
+def var_symbol_name(sym):
+    if var_is_wildcard(sym):
+        return sym[:-2]
+    return sym
+
 class vardef(object):
-    def __init__(self, nam, typ, rd, wr, wu, al, vlo, vhi):
-        self.nam = nam
+    def __init__(self, sym, typ, rd, wr, wu, al, vlo, vhi):
+        self.sym = sym
+        self.nam = var_symbol_name(sym)
         self.typ = typ
         self.rd = rd
         self.wr = wr
@@ -192,7 +166,7 @@ class vardef(object):
 
         # fo.write("\t{ \"%s\", %s,\n" % (nm, self.typ))
         fo.write("\tsym = VCC_MkSym(tl, \"%s\", SYM_MAIN," % self.nam)
-        if self.typ == "HEADER":
+        if var_is_wildcard(self.sym):
             fo.write(" SYM_NONE, %d, %d);\n" % (self.vlo, self.vhi))
             fo.write("\tAN(sym);\n")
             fo.write("\tsym->wildcard = vcc_Var_Wildcard;\n")
@@ -200,20 +174,23 @@ class vardef(object):
             fo.write(" SYM_VAR, %d, %d);\n" % (self.vlo, self.vhi))
         fo.write("\tAN(sym);\n")
         fo.write("\tsym->type = %s;\n" % self.typ)
-        fo.write("\tsym->eval = vcc_Eval_Var;\n")
+        if self.typ == "HEADER" and not var_is_wildcard(self.sym):
+            fo.write("\tsym->eval = vcc_Eval_ProtectedHeader;\n")
+        else:
+            fo.write("\tsym->eval = vcc_Eval_Var;\n")
 
-        if self.typ == "HEADER":
+        if var_is_wildcard(self.sym):
             fo.write('\tsym->rname = "HDR_')
             fo.write(self.nam.split(".")[0].upper())
             fo.write('";\n')
-        elif self.rd:
+        elif self.rd and self.typ != "HEADER":
             fo.write('\tsym->rname = "VRT_r_%s(ctx)";\n' % cnam)
             varproto("VCL_" + self.typ + " VRT_r_%s(VRT_CTX)" % cnam)
         fo.write("\tsym->r_methods =\n")
         restrict(fo, self.rd)
         fo.write(";\n")
 
-        if self.typ == "HEADER":
+        if var_is_wildcard(self.sym):
             fo.write('\tsym->lname = "HDR_')
             fo.write(self.nam.split(".")[0].upper())
             fo.write('";\n')
@@ -260,8 +237,6 @@ def parse_var(ln):
     l1 = ln.pop(0).split("``")
     assert len(l1) in (1, 3)
     vn = l1[0].strip()
-    if vn[-2:] == '.*':
-        vn = vn[:-2]
     if len(l1) == 3:
         vlo, vhi = parse_vcl(l1[1])
     else:
@@ -312,15 +287,15 @@ def parse_var_doc(fn):
         parse_var(l[n-2:m-1])
 
 stv_variables = (
-    ('free_space', 'BYTES', "0.", 'storage.<name>.free_space', """
+    ('free_space', 'BYTES', "0", 'storage.<name>.free_space', """
     Free space available in the named stevedore. Only available for
     the malloc stevedore.
     """),
-    ('used_space', 'BYTES', "0.", 'storage.<name>.used_space', """
+    ('used_space', 'BYTES', "0", 'storage.<name>.used_space', """
     Used space in the named stevedore. Only available for the malloc
     stevedore.
     """),
-    ('happy', 'BOOL', "0", 'storage.<name>.happy', """
+    ('happy', 'BOOL', "1", 'storage.<name>.happy', """
     Health status for the named stevedore. Not available in any of the
     current stevedores.
     """),
@@ -689,7 +664,7 @@ struct VCL_conf {
 	unsigned		nref;
 	const struct vpi_ref	*ref;
 
-	int			nsrc;
+	unsigned		nsrc;
 	unsigned		nsub;
 	const char		**srcname;
 	const char		**srcbody;
@@ -708,6 +683,22 @@ fo.close()
 
 #######################################################################
 
+fo = open(join(buildroot, "include/tbl/vcl_context.h"), "w")
+file_header(fo)
+
+fo.write("/*lint -save -e525 -e539 */\n")
+for i in returns:
+    fo.write("\nVCL_CTX(vcl_%s,%s)" % (i[0],i[0].upper()))
+fo.write("\nVCL_CTX(backend, TASK_B)")
+fo.write("\nVCL_CTX(client, TASK_C)")
+fo.write("\nVCL_CTX(housekeeping, TASK_H)")
+fo.write("/*lint -restore */\n")
+fo.write("\n")
+fo.write("\n#undef VCL_CTX")
+fo.write("\n")
+fo.close()
+
+#######################################################################
 
 def restrict(fo, spec):
     d = dict()
